@@ -1,6 +1,7 @@
 //! Different kinds of messages that can be encoded in channel cells.
 
 use super::{ChanCmd, RawCellBody, CELL_DATA_LEN};
+use std::convert::TryInto;
 use std::net::{IpAddr, Ipv4Addr};
 use tor_bytes::{self, Error, Readable, Reader, Result, Writer};
 
@@ -529,7 +530,7 @@ caret_int! {
         PROTOCOL = 1,
         /// Internal error.
         INTERNAL = 2,
-        /// Client sent a TRUNCATE commant.
+        /// Client sent a TRUNCATE command.
         REQUESTED = 3,
         /// Relay is hibernating and not accepting requests
         HIBERNATING = 4,
@@ -600,8 +601,8 @@ fn take_one_netinfo_addr(r: &mut Reader<'_>) -> Result<Option<IpAddr>> {
             (&mut bytes[..]).copy_from_slice(abody);
             Ok(Some(IpAddr::V6(bytes.into())))
         }
-        (0x04, _) => Ok(None), // TODO SPEC this is ignored.
-        (0x06, _) => Ok(None), // TODO SPEC this is ignored.
+        (0x04, _) => Ok(None),
+        (0x06, _) => Ok(None),
         (_, _) => Ok(None),
     }
 }
@@ -637,8 +638,12 @@ impl Body for Netinfo {
             .their_addr
             .unwrap_or_else(|| Ipv4Addr::UNSPECIFIED.into());
         enc_one_netinfo_addr(w, &their_addr);
-        assert!(self.my_addr.len() <= u8::MAX as usize);
-        w.write_u8(self.my_addr.len() as u8);
+        let n_addrs: u8 = self
+            .my_addr
+            .len()
+            .try_into()
+            .expect("Too many addrs in netinfo cell");
+        w.write_u8(n_addrs);
         for addr in self.my_addr.iter() {
             enc_one_netinfo_addr(w, addr);
         }
@@ -679,14 +684,19 @@ pub struct Versions {
 }
 impl Versions {
     /// Construct a new Versions message using a provided list of link
-    /// protocols
-    pub fn new<B>(vs: B) -> Self
+    /// protocols.
+    ///
+    /// Returns an error if the list of versions is too long.
+    pub fn new<B>(vs: B) -> crate::Result<Self>
     where
         B: Into<Vec<u16>>,
     {
         let versions = vs.into();
-        assert!(versions.len() < (std::u16::MAX / 2) as usize);
-        Self { versions }
+        if versions.len() < (std::u16::MAX / 2) as usize {
+            Ok(Self { versions })
+        } else {
+            Err(crate::Error::CantEncode)
+        }
     }
     /// Encode this VERSIONS cell in the manner expected for a handshake.
     ///
@@ -751,6 +761,20 @@ pub struct PaddingNegotiate {
     // XXXX is that right?
     ito_high_ms: u16,
 }
+impl PaddingNegotiate {
+    /// Create a new PaddingNegotiate message.
+    ///
+    /// If `start` is true, this is a message to enable padding. Otherwise
+    /// this is a message to disable padding.
+    pub fn new(start: bool, ito_low_ms: u16, ito_high_ms: u16) -> Self {
+        let command = if start { 2 } else { 1 };
+        Self {
+            command,
+            ito_low_ms,
+            ito_high_ms,
+        }
+    }
+}
 impl Body for PaddingNegotiate {
     fn into_message(self) -> ChanMsg {
         ChanMsg::PaddingNegotiate(self)
@@ -795,8 +819,12 @@ struct TorCert {
 /// encode a single TorCert `c` onto a Writer `w`.
 fn enc_one_tor_cert<W: Writer + ?Sized>(w: &mut W, c: &TorCert) {
     w.write_u8(c.certtype);
-    assert!(c.cert.len() <= u16::MAX as usize);
-    w.write_u16(c.cert.len() as u16);
+    let cert_len: u16 = c
+        .cert
+        .len()
+        .try_into()
+        .expect("Impossibly long certificate");
+    w.write_u16(cert_len);
     w.write_all(&c.cert[..]);
 }
 /// Try to extract a TorCert from the reader `r`.
@@ -873,8 +901,12 @@ impl Body for Certs {
         ChanMsg::Certs(self)
     }
     fn write_body_onto<W: Writer + ?Sized>(self, w: &mut W) {
-        assert!(self.certs.len() <= u8::MAX as usize);
-        w.write_u8(self.certs.len() as u8);
+        let n_certs: u8 = self
+            .certs
+            .len()
+            .try_into()
+            .expect("Too many certs to encode in cell.");
+        w.write_u8(n_certs);
         for c in self.certs.iter() {
             enc_one_tor_cert(w, c)
         }
@@ -1085,7 +1117,8 @@ impl<B: Body> From<B> for ChanMsg {
     }
 }
 
-// Helper: declare an Into implementation for cells that don't take a circid.
+/// Helper: declare a From<> implementation from message types for
+/// cells that don't take a circid.
 macro_rules! msg_into_cell {
     ($body:ident) => {
         impl From<$body> for super::ChanCell {
