@@ -7,6 +7,7 @@ use super::RelayCmd;
 use crate::chancell::msg::{DestroyReason, TAP_C_HANDSHAKE_LEN, TAP_S_HANDSHAKE_LEN};
 use crate::chancell::CELL_DATA_LEN;
 use caret::caret_int;
+use std::convert::TryInto;
 use std::net::{IpAddr, Ipv4Addr};
 use tor_bytes::{Error, Result};
 use tor_bytes::{Readable, Reader, Writeable, Writer};
@@ -200,7 +201,7 @@ impl Default for IpVersionPreference {
 /// If the exit decides to reject the Begin message, or if the TCP
 /// connection fails, the exit should send an End message.
 ///
-/// Clients should reject these messags.
+/// Clients should reject these messages.
 #[derive(Debug, Clone)]
 pub struct Begin {
     /// Ascii string describing target address
@@ -221,7 +222,7 @@ impl Begin {
             return Err(crate::Error::BadStreamAddress);
         }
         let mut addr = addr.to_string();
-        addr.make_ascii_lowercase(); // SPEC: the spec doesn't say to do this.
+        addr.make_ascii_lowercase();
         Ok(Begin {
             addr: addr.into_bytes(),
             port,
@@ -304,9 +305,31 @@ impl Data {
     pub const MAXLEN: usize = CELL_DATA_LEN - 11;
 
     /// Construct a new data cell.
-    pub fn new(inp: &[u8]) -> Self {
-        assert!(inp.len() <= Data::MAXLEN);
-        Data { body: inp.into() }
+    ///
+    /// Returns an error if `inp` is longer than [`Data::MAXLEN`] bytes.
+    pub fn new(inp: &[u8]) -> crate::Result<Self> {
+        if inp.len() > Data::MAXLEN {
+            return Err(crate::Error::CantEncode);
+        }
+        Ok(Self::new_unchecked(inp.into()))
+    }
+
+    /// Construct a new data cell, taking as many bytes from `inp`
+    /// as possible.
+    ///
+    /// Return the data cell, and a slice holding any bytes that
+    /// wouldn't fit (if any).
+    pub fn split_from(inp: &[u8]) -> (Self, &[u8]) {
+        let len = std::cmp::min(inp.len(), Data::MAXLEN);
+        let (data, remainder) = inp.split_at(len);
+        (Self::new_unchecked(data.into()), remainder)
+    }
+
+    /// Construct a new data cell from a provided vector of bytes.
+    ///
+    /// The vector _must_ have fewer than [`Data::MAXLEN`] bytes.
+    fn new_unchecked(body: Vec<u8>) -> Self {
+        Data { body }
     }
 }
 impl From<Data> for Vec<u8> {
@@ -417,7 +440,6 @@ impl Body for End {
     }
     fn decode_from_reader(r: &mut Reader<'_>) -> Result<Self> {
         if r.remaining() == 0 {
-            // TODO SPEC Document this behavior; tor does it too.
             return Ok(End {
                 reason: EndReason::MISC,
                 addr: None,
@@ -429,7 +451,6 @@ impl Body for End {
                 4 | 8 => IpAddr::V4(r.extract()?),
                 16 | 20 => IpAddr::V6(r.extract()?),
                 _ => {
-                    // TODO SPEC document this behavior.
                     // Ignores other message lengths.
                     return Ok(End { reason, addr: None });
                 }
@@ -606,13 +627,14 @@ impl Body for Sendme {
     }
     fn encode_onto(self, w: &mut Vec<u8>) {
         match self.digest {
-            // SPEC: we should be clear in the spec that this is what we
-            // do when linkauth is off.
             None => (),
             Some(mut x) => {
                 w.write_u8(1);
-                assert!(x.len() <= u16::MAX as usize);
-                w.write_u16(x.len() as u16);
+                let bodylen: u16 = x
+                    .len()
+                    .try_into()
+                    .expect("Too many bytes to encode in relay cell.");
+                w.write_u16(bodylen);
                 w.append(&mut x)
             }
         }
@@ -765,14 +787,14 @@ impl Body for Extend2 {
         })
     }
     fn encode_onto(self, w: &mut Vec<u8>) {
-        assert!(self.linkspec.len() <= std::u8::MAX as usize);
-        w.write_u8(self.linkspec.len() as u8);
+        let n_linkspecs: u8 = self.linkspec.len().try_into().expect("Too many linkspecs");
+        w.write_u8(n_linkspecs);
         for ls in self.linkspec.iter() {
             w.write(ls);
         }
         w.write_u16(self.handshake_type);
-        assert!(self.handshake.len() <= std::u16::MAX as usize);
-        w.write_u16(self.handshake.len() as u16);
+        let handshake_len: u16 = self.handshake.len().try_into().expect("Handshake too long");
+        w.write_u16(handshake_len);
         w.write_all(&self.handshake[..]);
     }
 }
@@ -810,13 +832,13 @@ impl Body for Extended2 {
         })
     }
     fn encode_onto(self, w: &mut Vec<u8>) {
-        assert!(self.handshake.len() <= std::u16::MAX as usize);
-        w.write_u16(self.handshake.len() as u16);
+        let handshake_len: u16 = self.handshake.len().try_into().expect("Handshake too long");
+        w.write_u16(handshake_len);
         w.write_all(&self.handshake[..]);
     }
 }
 
-/// A Truncated messsage is sent to the client when the remaining hops
+/// A Truncated message is sent to the client when the remaining hops
 /// of a circuit have gone away.
 ///
 /// NOTE: Current Tor implementations often treat Truncated messages and
@@ -906,7 +928,7 @@ impl Body for Resolve {
 }
 
 /// Possible response to a DNS lookup
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Eq, PartialEq)]
 #[non_exhaustive]
 pub enum ResolvedVal {
     /// We found an IP address
@@ -973,8 +995,8 @@ impl Writeable for ResolvedVal {
         match self {
             Hostname(h) => {
                 w.write_u8(RES_HOSTNAME);
-                assert!(h.len() <= std::u8::MAX as usize);
-                w.write_u8(h.len() as u8);
+                let h_len: u8 = h.len().try_into().expect("Hostname too long");
+                w.write_u8(h_len);
                 w.write_all(&h[..]);
             }
             Ip(IpAddr::V4(a)) => {
@@ -997,8 +1019,11 @@ impl Writeable for ResolvedVal {
             }
             Unrecognized(tp, v) => {
                 w.write_u8(*tp);
-                assert!(v.len() <= std::u8::MAX as usize);
-                w.write_u8(v.len() as u8);
+                let v_len: u8 = v
+                    .len()
+                    .try_into()
+                    .expect("Unrecognized resolved entry too long");
+                w.write_u8(v_len);
                 w.write_all(&v[..]);
             }
         }
@@ -1039,6 +1064,17 @@ impl Resolved {
     pub fn add_answer(&mut self, answer: ResolvedVal, ttl: u32) {
         self.answers.push((answer, ttl));
     }
+
+    /// Consume this Resolved message, returning a vector of the
+    /// answers and TTL values that it contains.
+    ///
+    /// Note that actually relying on these TTL values can be
+    /// dangerous in practice, since the relay that sent the cell
+    /// could be lying in order to cause more lookups, or to get a
+    /// false answer cached for longer.
+    pub fn into_answers(self) -> Vec<(ResolvedVal, u32)> {
+        self.answers
+    }
 }
 impl Body for Resolved {
     fn into_message(self) -> RelayMsg {
